@@ -12,6 +12,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -23,8 +31,9 @@
  *
  */
 
-#define NO_IMPORT_PYGOBJECT
-#include "pygobject.h"
+#include "config.h"
+
+#include <pygobject.h>
 #include <pygtk/pygtk.h>
 
 #include "override_common.h"
@@ -86,7 +95,7 @@ _helper_wrap_boxed_glist (const GList *list,
         return NULL;
     }
     for (tmp = list; tmp != NULL; tmp = tmp->next) {
-        PyObject *py_obj = pyg_boxed_new(boxed_type, G_OBJECT(tmp->data), copy_boxed, own_ref);
+        PyObject *py_obj = pyg_boxed_new(boxed_type, tmp->data, copy_boxed, own_ref);
 
         if (py_obj == NULL) {
             Py_DECREF(py_list);
@@ -108,7 +117,7 @@ _helper_wrap_string_glist (const GList *list)
         return NULL;
     }
     for (tmp = list; tmp != NULL; tmp = tmp->next) {
-        PyObject *str_obj =  PyString_FromString ((char*)tmp->data);
+        PyObject *str_obj =  PyString_FromString (tmp->data);
 
         if (str_obj == NULL) {
             Py_DECREF(py_list);
@@ -163,7 +172,51 @@ _helper_unwrap_pointer_pylist (PyObject *py_list, GType type)
 }
 
 GList *
-_helper_unwrap_string_pylist (PyObject *py_list)
+_helper_unwrap_boxed_pylist (PyObject *py_list, GType type)
+{
+	int size, i;
+	GList *list = NULL;
+
+	size = PyList_Size (py_list);
+	for (i = 0; i < size; i++) {
+        	PyObject *py_ptr;
+        	gpointer ptr;
+
+		py_ptr = PyList_GetItem (py_list, i);
+		if (!pyg_boxed_check (py_ptr, type)) {
+			g_list_free (list);
+			return NULL;
+		}
+		ptr = pyg_boxed_get (py_ptr, void);
+		list = g_list_prepend (list, ptr);
+	}
+
+	list = g_list_reverse (list);
+	return list;
+}
+
+GList *
+_helper_unwrap_gobject_pylist (PyObject *py_list)
+{
+	int size, i;
+	GList *list = NULL;
+
+	size = PyList_Size (py_list);
+	for (i = 0; i < size; i++) {
+        	PyObject *py_ptr;
+        	gpointer ptr;
+
+		py_ptr = PyList_GetItem (py_list, i);
+		ptr = pygobject_get (py_ptr);
+		list = g_list_prepend (list, ptr);
+	}
+
+	list = g_list_reverse (list);
+	return list;
+}
+
+GList *
+_helper_unwrap_string_pylist (PyObject *py_list, gboolean duplicate)
 {
     int size, i;
     GList *list = NULL;
@@ -175,10 +228,88 @@ _helper_unwrap_string_pylist (PyObject *py_list)
 
 	py_str = PyList_GetItem (py_list, i);
 	str = PyString_AsString (py_str);
-	list = g_list_prepend (list, str);
+	if (duplicate)
+		list = g_list_prepend (list, g_strdup (str));
+	else
+		list = g_list_prepend (list, str);
     }
 
     list = g_list_reverse (list);
     return list;
 }
 
+GPtrArray*
+_helper_unwrap_boxed_gptrarray (PyObject *list, GType type)
+{
+	int size, i;
+	GPtrArray *array;
+
+	array = g_ptr_array_new ();
+	size = PyList_Size (list);
+	for (i = 0; i < size; i++) {
+		PyObject *py_boxed;
+		gpointer boxed;
+
+		py_boxed = PyList_GetItem (list, i);
+		if (!pyg_boxed_check (py_boxed, type)) {
+			g_ptr_array_free (array, FALSE);
+			return NULL;
+		}
+
+		boxed = pyg_boxed_get (py_boxed, void);
+		g_ptr_array_add (array, boxed);
+	}
+
+	return array;
+}
+
+/* query model sorting stuff */
+void
+_rhythmdb_query_model_sort_data_free (PyRhythmDBQueryModelSortData *data)
+{
+	PyGILState_STATE __py_state;
+	__py_state = pyg_gil_state_ensure();
+
+	Py_DECREF (data->func);
+	Py_DECREF (data->data);
+	g_free (data);
+
+	pyg_gil_state_release(__py_state);
+}
+
+int
+_rhythmdb_query_model_sort_func (RhythmDBEntry *a, RhythmDBEntry *b, PyRhythmDBQueryModelSortData *data)
+{
+	PyObject *args;
+	PyObject *py_result;
+	PyObject *py_a, *py_b;
+	int result;
+	PyGILState_STATE __py_state;
+
+	__py_state = pyg_gil_state_ensure();
+
+	py_a = pyg_boxed_new (RHYTHMDB_TYPE_ENTRY, a, FALSE, FALSE);
+	py_b = pyg_boxed_new (RHYTHMDB_TYPE_ENTRY, b, FALSE, FALSE);
+	if (data->data)
+		args = Py_BuildValue ("(OOO)", py_a, py_b, data->data);
+	else
+		args = Py_BuildValue ("(OO)", py_a, py_b);
+
+	Py_DECREF (py_a);
+	Py_DECREF (py_b);
+
+	py_result = PyEval_CallObject (data->func, args);
+	Py_DECREF (args);
+
+	if (!py_result) {
+		PyErr_Print();
+		return NULL;
+	}
+	result = PyInt_AsLong (py_result);
+
+	Py_DECREF (py_result);
+	pyg_gil_state_release(__py_state);
+
+	return result;
+}
+/* end query model sorting stuff */
