@@ -54,16 +54,15 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
-#if !GTK_CHECK_VERSION(2,13,1)
+#if !GTK_CHECK_VERSION(2,14,0)
 #include <libgnome/gnome-help.h>
 #endif
 
 #include "rb-file-helpers.h"
 #include "rb-shell-preferences.h"
 #include "rb-source.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
@@ -92,8 +91,6 @@ static void rb_shell_preferences_toolbar_style_cb (GtkComboBox *box,
 						   RBShellPreferences *preferences);
 static void rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
 						    RBShellPreferences *preferences);
-static void rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
-						     RBShellPreferences *preferences);
 static void rb_shell_preferences_transition_duration_cb (GtkRange *range,
 							 RBShellPreferences *preferences);
 static void rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
@@ -126,7 +123,6 @@ struct RBShellPreferencesPrivate
 	GtkWidget *location_check;
 
 	GtkWidget *xfade_backend_check;
-	GtkWidget *album_crossfade_check;
 	GtkWidget *transition_duration;
 	GtkWidget *network_buffer_size;
 
@@ -157,9 +153,9 @@ help_cb (GtkWidget *widget,
 {
 	GError *error = NULL;
 
-#if GTK_CHECK_VERSION(2,13,1)
+#if GTK_CHECK_VERSION(2,14,0)
 	gtk_show_uri (gtk_widget_get_screen (widget),
-		      "ghelp:rhythmbox",
+		      "ghelp:rhythmbox?prefs",
 		      gtk_get_current_event_time (),
 		      &error);
 #else
@@ -179,7 +175,7 @@ static void
 rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 {
 	GtkWidget *tmp;
-	GladeXML *xml;
+	GtkBuilder *builder;
 
 	shell_preferences->priv = RB_SHELL_PREFERENCES_GET_PRIVATE (shell_preferences);
 
@@ -216,54 +212,52 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (shell_preferences)->vbox), 2);
 	gtk_dialog_set_has_separator (GTK_DIALOG (shell_preferences), FALSE);
 
-	xml = rb_glade_xml_new ("general-prefs.glade",
-				"general_vbox",
-				shell_preferences);
+	builder = rb_builder_load ("general-prefs.ui", shell_preferences);
 
-	rb_glade_boldify_label (xml, "visible_columns_label");
+	rb_builder_boldify_label (builder, "visible_columns_label");
 
 	/* Columns */
 	shell_preferences->priv->artist_check =
-		glade_xml_get_widget (xml, "artist_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "artist_check"));
 	shell_preferences->priv->album_check =
-		glade_xml_get_widget (xml, "album_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "album_check"));
 	shell_preferences->priv->genre_check =
-		glade_xml_get_widget (xml, "genre_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "genre_check"));
 	shell_preferences->priv->duration_check =
-		glade_xml_get_widget (xml, "duration_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "duration_check"));
 	shell_preferences->priv->track_check =
-		glade_xml_get_widget (xml, "track_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "track_check"));
 	shell_preferences->priv->rating_check =
-		glade_xml_get_widget (xml, "rating_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "rating_check"));
 	shell_preferences->priv->play_count_check =
-		glade_xml_get_widget (xml, "play_count_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "play_count_check"));
 	shell_preferences->priv->last_played_check =
-		glade_xml_get_widget (xml, "last_played_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "last_played_check"));
 	shell_preferences->priv->quality_check =
-		glade_xml_get_widget (xml, "quality_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "quality_check"));
 	shell_preferences->priv->year_check =
-		glade_xml_get_widget (xml, "year_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "year_check"));
 	shell_preferences->priv->first_seen_check =
-		glade_xml_get_widget (xml, "first_seen_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "first_seen_check"));
 	shell_preferences->priv->location_check =
-		glade_xml_get_widget (xml, "location_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "location_check"));
 
 	/* browser options */
-	rb_glade_boldify_label (xml, "browser_views_label");
+	rb_builder_boldify_label (builder, "browser_views_label");
 
-	tmp = glade_xml_get_widget (xml, "library_browser_views_radio");
+	tmp = GTK_WIDGET (gtk_builder_get_object (builder, "library_browser_views_radio"));
 	shell_preferences->priv->browser_views_group =
 		g_slist_reverse (g_slist_copy (gtk_radio_button_get_group
 					       (GTK_RADIO_BUTTON (tmp))));
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
-				  glade_xml_get_widget (xml, "general_vbox"),
+				  GTK_WIDGET (gtk_builder_get_object (builder, "general_vbox")),
 				  gtk_label_new (_("General")));
 
 	/* toolbar button style */
-	rb_glade_boldify_label (xml, "toolbar_style_label");
+	rb_builder_boldify_label (builder, "toolbar_style_label");
 	shell_preferences->priv->toolbar_style_menu =
-		glade_xml_get_widget (xml, "toolbar_style_menu");
+		GTK_WIDGET (gtk_builder_get_object (builder, "toolbar_style_menu"));
 	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (shell_preferences->priv->toolbar_style_menu),
 					      rb_combo_box_hyphen_separator_func,
 					      NULL, NULL);
@@ -274,37 +268,27 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	eel_gconf_notification_add (CONF_UI_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
 				    shell_preferences);
-	g_object_unref (xml);
+	g_object_unref (builder);
 
 	/* playback preferences */
-	xml = rb_glade_xml_new ("playback-prefs.glade",
-				"playback_prefs_box",
-				shell_preferences);
+	builder = rb_builder_load ("playback-prefs.ui", shell_preferences);
 
-	rb_glade_boldify_label (xml, "backend_label");
-	rb_glade_boldify_label (xml, "crossfade_type_label");
-	rb_glade_boldify_label (xml, "duration_label");
-	rb_glade_boldify_label (xml, "buffer_size_label");
+	rb_builder_boldify_label (builder, "backend_label");
+	rb_builder_boldify_label (builder, "duration_label");
+	rb_builder_boldify_label (builder, "buffer_size_label");
 
 	shell_preferences->priv->xfade_backend_check =
-		glade_xml_get_widget (xml, "use_xfade_backend");
-	shell_preferences->priv->album_crossfade_check =
-		glade_xml_get_widget (xml, "album_check");
+		GTK_WIDGET (gtk_builder_get_object (builder, "use_xfade_backend"));
 	shell_preferences->priv->transition_duration =
-		glade_xml_get_widget (xml, "duration");
+		GTK_WIDGET (gtk_builder_get_object (builder, "duration"));
 	shell_preferences->priv->network_buffer_size =
-		glade_xml_get_widget (xml, "network_buffer_size");
+		GTK_WIDGET (gtk_builder_get_object (builder, "network_buffer_size"));
 
 	g_signal_connect_object (shell_preferences->priv->xfade_backend_check,
 				 "toggled",
 				 G_CALLBACK (rb_shell_preferences_player_backend_cb),
 				 shell_preferences, 0);
 
-	g_signal_connect_object (shell_preferences->priv->album_crossfade_check,
-				 "toggled",
-				 G_CALLBACK (rb_shell_preferences_album_crossfade_cb),
-				 shell_preferences, 0);
-
 	g_signal_connect_object (shell_preferences->priv->transition_duration,
 				 "value-changed",
 				 G_CALLBACK (rb_shell_preferences_transition_duration_cb),
@@ -316,9 +300,9 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				 shell_preferences, 0);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
-				  glade_xml_get_widget (xml, "playback_prefs_box"),
+				  GTK_WIDGET (gtk_builder_get_object (builder, "playback_prefs_box")),
 				  gtk_label_new (_("Playback")));
-	g_object_unref (xml);
+	g_object_unref (builder);
 
 	eel_gconf_notification_add (CONF_PLAYER_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
@@ -613,9 +597,6 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 	b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
 
-	b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
-
 	duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
 	gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
 
@@ -670,8 +651,6 @@ update_playback_prefs_sensitivity (RBShellPreferences *preferences)
 
 	backend = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences->priv->xfade_backend_check));
 
-	gtk_widget_set_sensitive (preferences->priv->album_crossfade_check, backend);
-	gtk_widget_set_sensitive (preferences->priv->network_buffer_size, backend);
 	gtk_widget_set_sensitive (preferences->priv->transition_duration, backend);
 }
 
@@ -685,17 +664,6 @@ rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
 			       gtk_toggle_button_get_active (button));
 }
 
-static void
-rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
-					 RBShellPreferences *preferences)
-{
-	/* sense is inverted here because the wording sounded more natural
-	 * this way.  "[ ] Don't do x" sucks.
-	 */
-	eel_gconf_set_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK,
-			       !gtk_toggle_button_get_active (button));
-}
-
 static void
 rb_shell_preferences_transition_duration_cb (GtkRange *range,
 					     RBShellPreferences *preferences)
